#include <iostream>
#include <iomanip>
#include <cmath>
#include <complex>
#include "gmpfrxx.h"
#include "relative_error.h"
#include "complex_gmpfrxx.h"

using namespace std;

int main()
{
	complex<double> a, b, c;
	complex<double> d, sol[2];

	mpfr_class::set_dprec_dec(50);
	complex<mpfr_class> ga, gb, gc;
	complex<mpfr_class> gd, gsol[2];

	cout << "a = ";
	cin >> ga;
	cout << "b = ";
	cin >> gb;
	cout << "c = ";
	cin >> gc;

	a.real() = ga.real().get_d(); a.imag() = ga.imag().get_d();
	b.real() = gb.real().get_d(); b.imag() = gb.imag().get_d();
	c.real() = gc.real().get_d(); c.imag() = gc.imag().get_d();

	cout << "input equation: ";
	cout << a << " * x^2 + ";
	cout << b << " * x   + ";
	cout << c << " = 0 " << endl;

	if(a == 0.0)
	{
		if(b == 0.0)
		{
			cout << "Cannot solve!" << endl;
			return -1;
		}
		sol[0] = -c / b;

		cout << "Solution: " << sol[0] << endl;

		return 0;
	}

	d = b * b - 4.0 * a * c;

	sol[0] = (-b + sqrt(d)) / (2.0 * a);
	sol[1] = (-b - sqrt(d)) / (2.0 * a);

	gd = gb * gb - complex<mpfr_class>(4UL, 0UL) * ga * gc;
	gsol[0] = (-gb + sqrt(gd)) / (complex<mpfr_class>(2UL, 0UL) * ga);
	gsol[1] = (-gb - sqrt(gd)) / (complex<mpfr_class>(2UL, 0UL) * ga);

	cout.precision(50);
	cout << "Solutions: " << sol[0] << ", " << sol[1] << endl;
	cout << "Solutions: " << gsol[0] << ", " << gsol[1] << endl;

	cout.precision(5);
	cout << "Relative Errors in sol[0]: " << relative_error(sol[0].real(), gsol[0].real()) << ", " << relative_error(sol[0].imag(), gsol[0].imag()) << endl;
	cout << "Relative Errors in sol[1]: " << relative_error(sol[1].real(), gsol[1].real()) << ", " << relative_error(sol[1].imag(), gsol[1].imag()) << endl;

	return 0;
}
